from django.conf import settings


class OrderCreatorApprovalMixin(object):

    def update_line_approval_extra_fields(self, order, basket_line, extra_line_fields):

        """
            Updates line additional data with stuff related to order approval.
            Can be used in a class inheriting from oscar's OrderCreator
            in the 'create_line_models' method:

        """

        if basket_line.product.requires_approval:
            if extra_line_fields is None:
                extra_line_fields = {}
        extra_line_fields['status'] = settings.OSCAR_LINE_APPROVAL_STATUS
        order.status = settings.OSCAR_ORDER_APPROVAL_STATUS
        order.save()
        return extra_line_fields




from abstract_models import AbstractOrderLineApprovalLog


class OrderLineApprovalLog(AbstractOrderLineApprovalLog):

    pass

from .signals import order_line_approved, order_line_rejected
from .receivers import receive_order_line_approved, receive_order_line_rejected

order_line_approved.connect(receive_order_line_approved)
order_line_rejected.connect(receive_order_line_rejected)



from django.views import generic
from django.http import HttpResponseRedirect
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from django.db.models import Q, get_model
from django.contrib.auth.models import User

from oscar.apps.dashboard.users import forms

from oscar_approval.apps.approval.models import OrderLineApprovalLog


class ApproverManagementView(generic.ListView):

    template_name = 'oscar_approval/dashboard/approver_management.html'
    model = get_model(*settings.AUTH_PROFILE_MODULE.rsplit('.', 1))
    form_class = forms.UserSearchForm

    def get_queryset(self):
        qs = self.model.objects.none()

        if self.form.is_bound:
            cd = self.form.cleaned_data
            if not cd['email'] and not cd['name']:
                return self.model.objects.none()

            qs = (self.model.objects.all()
                        .select_related('user'))

            if cd['email']:
                qs = qs.filter(user__email__startswith=cd['email'])

            if cd['name']:
                parts = cd['name'].split()
                if len(parts) == 2:
                    qs = qs.filter(Q(user__first_name__istartswith=parts[0]) |
                                   Q(user__last_name__istartswith=parts[1])).distinct()
                else:
                    qs = qs.filter(Q(user__first_name__istartswith=cd['name']) |
                                   Q(user__last_name__istartswith=cd['name'])).distinct()
        return qs

    def get_context_data(self, *args, **kwargs):
        ctx = super(ApproverManagementView, self).get_context_data(*args, 
                                                                   **kwargs)
        ctx['form'] = getattr(self, 'form', None) or self.form_class()
        ctx['approvers'] = (self.model.objects.filter(is_order_approver=True)
                                    .select_related('user'))
        return ctx

    def get(self, request, *args, **kwargs):
        self.form = self.form_class(request.GET or None)

        if not self.form.is_valid() and self.form.is_bound:
            ctx = self.get_context_data()
            return self.render_to_response(ctx)

        return super(ApproverManagementView, self).get(request, *args, **kwargs)


class ApproverUpdateView(generic.UpdateView):

    model = get_model(*settings.AUTH_PROFILE_MODULE.rsplit('.', 1))

    def post(self, request, *args, **kwargs):
        is_approver = bool(kwargs['is_approver'])

        profile = self.get_object()
        profile.is_order_approver = is_approver
        profile.save()

        messages.success(request, _(self.get_success_message(is_approver)))

        return HttpResponseRedirect(self.get_success_url())

    def get_success_message(self, is_approver):
        if is_approver:
            msg = 'Approver has been added.'
        else:
            msg = 'Approver has been removed.'
        return msg

    def get_success_url(self):
        return reverse('dashboard:approver-management')


class EventLogView(generic.ListView):
    
    model = OrderLineApprovalLog
    template_name = 'oscar_approval/dashboard/event_log_list.html'
    paginate_by = 20


    def get_queryset(self, *args, **kwargs):
        qs = super(EventLogView, self).get_queryset(*args, **kwargs)
        qs = (qs.select_related('line__order__user', 'line__product', 'user')
                .order_by('-event_date'))
        return qs



from django import forms
from django.utils.translation import ugettext_lazy as _


class ApprovalSearchForm(forms.Form):

    query = forms.CharField(label=_('Search term'), required=False,
                            help_text=_('Search by product name'))

    date_from = forms.DateField(required=False, label=_("From"))
    date_to = forms.DateField(required=False, label=_("To"))



from functools import wraps

from django.contrib import messages
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _


def order_approver_required(view_func):
    @wraps(view_func)
    def _check_approver(request, *args, **kwargs):
        if request.user.is_active and request.user.get_profile().is_order_approver:
            return view_func(request, *args, **kwargs)

        messages.warning(request, _("You don't have permission to access this page"))
        return HttpResponseRedirect(reverse('customer:summary'))

    return _check_approver



from django.db import models
from django.utils.translation import ugettext_lazy as _


class AbstractProfile(models.Model):

    is_order_approver = models.BooleanField(_('Has permissions to approve orders'),
                                            default=False)

    class Meta:
        abstract = True



from django.conf.urls.defaults import patterns, url
from django.utils.translation import ugettext_lazy as _

from oscar.core.application import Application
from oscar.apps.dashboard.nav import register, Node

from . import views

node = Node(_('Approval'))
node.add_child(Node(_('Approver management'), 'dashboard:approver-management'))
node.add_child(Node(_('Event log'), 'dashboard:approval-event-log'))
register(node, 150)


class ApprovalDashboardApplication(Application):
    name = None
    approvers_view = views.ApproverManagementView
    approver_update_view = views.ApproverUpdateView
    event_log_view = views.EventLogView

    def get_urls(self):
        urlpatterns = patterns('',
            url(r'^approver-management/$', self.approvers_view.as_view(),
                name='approver-management'),
            url(r'^approver-management/add-approver/(?P<pk>\d+)/$',
                self.approver_update_view.as_view(),
                name='approver-add',
                kwargs={'is_approver': True}),
            url(r'^approver-management/remove-approver/(?P<pk>\d+)/$',
                self.approver_update_view.as_view(),
                name='approver-remove',
                kwargs={'is_approver': False}),
            url(r'^event-log/$',
                self.event_log_view.as_view(),
                name='approval-event-log'),
        )
        return self.post_process_urls(urlpatterns)


application = ApprovalDashboardApplication()



from datetime import timedelta

from django.views import generic
from django.db.models import get_model
from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from . import signals, forms

Line = get_model('order', 'Line')


class OrderLineApprovalListView(generic.ListView):

    template_name = 'oscar_approval/order_line_approval_list.html'
    search_form_class = forms.ApprovalSearchForm
    model = Line
    paginate_by = 10

    def get_queryset(self):
        qs = self.get_base_query_set()

        self.form = self.search_form_class(self.request.GET)
        if not self.form.is_valid():
            return qs

        q = self.form.cleaned_data['query']
        if q:
            qs = self.filter_by_text_query(q, qs)

        date_from = self.form.cleaned_data['date_from']
        if date_from:
            qs = qs.filter(order__date_placed__gte=date_from)

        date_to = self.form.cleaned_data['date_to']
        if date_to:
            qs = qs.filter(order__date_placed__lt=date_to + timedelta(days=1))

        return qs

    def get_base_query_set(self):
        return self.model.objects.filter(
            status=settings.OSCAR_LINE_APPROVAL_STATUS) \
            .select_related('product', 'order') \
            .order_by('-order__date_placed')

    def filter_by_text_query(self, query, qs):
        qs = qs.filter(product__title__icontains=query)
        return qs

    def get_context_data(self, *args, **kwargs):
        ctx = super(OrderLineApprovalListView, self).get_context_data(*args,
                                                                      **kwargs)
        form = self.form if hasattr(self, 'form') else self.search_form()
        ctx['form'] = form
        return ctx


class OrderLineApproveView(generic.UpdateView):

    model = Line

    def post(self, request, *args, **kwargs):
        signals.order_line_approved.send(sender=self,
                                         line=self.get_object(),
                                         user=request.user)
        messages.success(request, _('Line purchase has been approved.'))
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse('approval:order-line-approval-list')


class OrderLineRejectView(generic.UpdateView):

    model = Line

    def post(self, request, *args, **kwargs):
        signals.order_line_rejected.send(sender=self,
                                         line=self.get_object(),
                                         user=request.user)
        messages.success(request, _('Line purchase has been rejected.'))
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse('approval:order-line-approval-list')



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User


APPROVE, REJECT = 'Approve', 'Reject'


class AbstractOrderLineApprovalLog(models.Model):

    EVENT_TYPE_CHOICES = (
        (APPROVE, _("Item approved")),
        (REJECT, _("Item rejected")),
    )

    line = models.ForeignKey('order.Line')
    user = models.ForeignKey(User)
    event_date = models.DateTimeField(auto_now_add=True)
    event_type = models.CharField(max_length=30)

    class Meta:
        abstract = True


from django.views import generic
from django.db.models import get_model
from django.conf import settings

Line = get_model('order', 'Line')


class AccountSummaryApprovalMixin(object):

    def get_account_approval_context_data(self):
        ctx = {
            'approval_item_number': (Line.objects.filter(
                                    status=settings.OSCAR_LINE_APPROVAL_STATUS)
                                    .count())
        }
        return ctx



from django.conf import settings


def receive_order_placed(sender, order, user, **kwargs):
    authorisation_required = False
    for line in order.lines.all():
        if line.product.requires_approval:
            line.status = settings.OSCAR_LINE_APPROVAL_STATUS
            line.save()
            authorisation_required = True

    if authorisation_required:
        order.status = settings.OSCAR_ORDER_APPROVAL_STATUS
        order.save()



from django.db.models import get_model

from .abstract_models import APPROVE, REJECT


def receive_order_line_approved(sender, line, user, **kwargs):
    OrderLineApprovalLog = get_model('approval', 'OrderLineApprovalLog')
    OrderLineApprovalLog.objects.create(line=line, user=user, event_type=APPROVE)


def receive_order_line_rejected(sender, line, user, **kwargs):
    OrderLineApprovalLog = get_model('approval', 'OrderLineApprovalLog')
    OrderLineApprovalLog.objects.create(line=line, user=user, event_type=REJECT)



from django.conf.urls.defaults import patterns, url

from oscar.core.application import Application

from . import views, decorators


class ApprovalApplication(Application):
    name = 'approval'
    approval_list_view = views.OrderLineApprovalListView
    approve_view = views.OrderLineApproveView
    reject_view = views.OrderLineRejectView

    def get_urls(self):
        urlpatterns = patterns('',
            url(r'^order-line/list/$',
                self.approval_list_view.as_view(),
                name='order-line-approval-list'),
            url(r'^order-line/(?P<pk>\d+)/approve/$',
                self.approve_view.as_view(),
                name='order-line-approve'),
            url(r'^order-line/(?P<pk>\d+)/reject/$',
                self.reject_view.as_view(),
                name='order-line-reject'),
        )
        return self.post_process_urls(urlpatterns)

    def get_url_decorator(self, url_name):
        return decorators.order_approver_required


application = ApprovalApplication()



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name='django-oscar-approval',
    version='0.1',
    url='https://github.com/tangentlabs/django-oscar-approval',
    author="Tangentlabs",
    author_email="tangentlabs@tangentlabs.co.uk",
    description="Order approval module for django-oscar",
    license='BSD',
    keywords="order approval, order authorisation",
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Unix',
        'Programming Language :: Python',
    ],
    install_requires=[
        'django-oscar>=0.4,<0.6',
    ])



from django.db import models
from django.utils.translation import ugettext_lazy as _


class AbstractProduct(models.Model):

    requires_approval = models.BooleanField(_("Requires authorisation"), default=False)

    class Meta:
        abstract = True
